require "sketchup.rb" 
require "langhandler.rb"
require "FileUtils"
	
podium_folder_path_locale = Sketchup.find_support_file('Plugins/SU_Podium_V26/locale/')
str=File.join(podium_folder_path_locale, "podium_tools")
locale=Sketchup.get_locale.upcase
strdef=str+"_EN-US.strings"
strloc=str+"_#{locale}.strings"

if File.exist?(strloc)
	strfil=strloc
else
	strfil=strdef
end
	  
strs=IO.read(strfil)
stra=strs.split("==")
$strh={}
0.step(stra.length-2, 2){|i|
	key=stra[i].gsub(/[[:space:]]/, '')
	next if key.empty? || stra[i+1].empty?
	$strh[key]=stra[i+1]
}
	
(def $strh.[](key); self.GetString(key); end) unless $strh.respond_to?(:[])

class Podium_Tools
	def aboutPlugin
		@pluginVersion = ""
		UI.messagebox("#{$strh['Version']}" + ": " + @pluginVersion)
	end

	def modelTitle()
		title = Sketchup.active_model.title
	
		if(title == "")
			title = "untitled.skp"
		else
			title = Sketchup.active_model.title + ".skp"
		end

		title = "#{$strh['AnalysisResults']} - " + title

		return title
    end
	
	@@webDialog = UI::WebDialog.new(Podium_Tools.new.modelTitle(), true, "new value", 900, 600, 200, 200, true)
	
	def go
		@m_MAC = ( Object::RUBY_PLATFORM =~ /darwin/i ? true : false )
		@m_WIN = ( (Object::RUBY_PLATFORM =~ /mswin/i || Object::RUBY_PLATFORM =~ /mingw/i) ? true : false )

		@light_count = 0
		@off_light_count = 0
		@hidden_light_count = 0
		@omni_count  = 0
		@spot_count  = 0
		@off_spot_count = 0
		@hidden_spot_count = 0
		@mats_count  = 0
		@mats_name   = ""
		@mats_ext_wrong = ""
		@mats_crash   = ""
		message = ""
		@highIntensityLem = ""
		@normalLemMaterialsNumber = 0
		@unexplodedImageNumber = 0
		@hiddenUnexplodedImageNumber = 0
		@msgWrongRenderNumber = 0
		@hiddenMsgWrongRenderNumber = 0
		@msgDTRNumber = 0
		@hiddenMsgDTRNumber = 0
		@noExtensionNumber = 0
		@hiddenNoExtensionNumber = 0
		@wrongExtNumber = 0
		@hiddenWrongExtNumber = 0
		@crashMatsNumber = 0
		@hiddenCrashMatsNumber = 0
		@flippedNumber = 0
		@hiddenFlippedNumber = 0
		
		@highIntesityLemNumber = 0
		@crashHighIntensityLemNumber = 0
		@crashHiddenHighIntensityLemNumber = 0
		@hiddenLEMNumber  = 0
		@lem_faces = 0
		@lemMaterialsNumber = 0
		@bumpMapMaterialsNumber = 0				
		@hiddenBumpMapMaterialsNumber = 0
		@bumpMapMaterials = ""
		
		@supported_textures = ["bmp","jpg","jpeg","png"]
		
		@msgWrongRender = String.new()
		@msgDTR = Array.new()
		@LEM = Array.new()
		@HIDDEN_LEM = Array.new()
		@HI_LEM = Array.new()
		@NORMAL_LEM = Array.new()
		@refl = Array.new()
		@refra = Array.new()
		@auto = Array.new()
		@podium_materials = Array.new()
		@mats = Hash.new(0)
		@hidden_mats = Hash.new(0)
		
		# For reset materials dialog
		@reset_refractions = true
		@reset_LEM = true
		@reset_blur = true
		@reset_all = true

		# general info
		@number_of_ComponentInstances = 0
		@number_of_Groups = 0
		
		# Collect data
		faces_check
		entities_check
		material_check
		wrongRender_unkownSymbol_check
		
		automatic_materials
		status "#{$strh["Done"]}"	
		
		# Present
		message += "#{$strh['SUPodiummodelanalysis']}" + ":<br><br>"
		message += "#{$strh['SUMMARY']}" + "<br>========================<br>"
		message += "#{$strh['LightSources']}" + ":<br>"
		
		#message += "&nbsp;%d point %s, %d %s turned off. <br>" % [@light_count, @light_count==1 ? "light" : "lights", @off_light_count, @off_light_count==1 ? "is" : "are" ]
		message +="&nbsp;%d " % [@light_count]
		if(@light_count==1)
			message+= "#{$strh['pointlight']}" + ", %d " % [@off_light_count]
		else
			message+= "#{$strh['pointlights']}" + ", %d " % [@off_light_count]
		end
		
		if(@off_light_count==1)
			message+= "#{$strh['isturnedoff']}" + ". <br>"
		else
			message+= "#{$strh['areturnedoff']}" + ". <br>"
		end
		
		#message += "&nbsp;%d spot light%s, %d %s turned off. <br>" % [@spot_count, @spot_count==1 ? "" : "s", @off_spot_count, @off_spot_count==1 ? "is" : "are" ]
		message += "&nbsp;%d " % [@spot_count]
		if(@spot_count==1)
			message+= "#{$strh['spotlight']}" + ", %d " % [@off_spot_count]
			else
			message+= "#{$strh['spotlights']}" + ", %d " % [@off_spot_count]
		end
		
		if(@off_spot_count==1)
			message+= "#{$strh['isturnedoff']}" + ". <br>"
		else
			message+= "#{$strh['areturnedoff']}" + ". <br>"
		end
		
		# calculate materials
		lemMaterials = ""
		
		lemMaterials = ""
		@mats.each {|key, value| 
			mat = Sketchup.active_model.materials[key]
			if (@LEM.include?(mat))
				@lem_faces += value.to_i
				lemMaterials += print_material(key, value.to_i)
			end
			if (@podium_materials.include?(key))
				@auto += key
			end
		}
		
		hiddenLemMaterialsNumber = 0
		@hidden_mats.each {|key, value| 
			mat = Sketchup.active_model.materials[key]
			if (@LEM.include?(mat))
				hiddenLemMaterialsNumber += 1
				lemMaterials += print_material(key, value.to_i, "hidden_text_button")
			end
		}
		
		message += "&nbsp;%d "                                  % [@lemMaterialsNumber]        + "#{$strh['LEMmaterial']}"       + "<br>"
		message += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%d "    % [@normalLemMaterialsNumber]  + "#{$strh['NormalLEM']}"         + "<br>"
		message += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%d "    % [@hiddenLEMNumber]           + "#{$strh['HiddenLEM']}"         + "<br>"
		message += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%d "    % [@highIntesityLemNumber]     + "#{$strh['HighIntensityLEM']}"  + "<br>"
		
		# reflective materials
		reflectiveMaterialsNumber = 0
		reflectiveMaterials = ""
		@mats.each {|key, value| 
			mat = Sketchup.active_model.materials[key]
			if (@refl.include?(mat))
				reflectiveMaterialsNumber += 1
				reflectiveMaterials += print_material(key, value.to_i)
			end	
		}
		hiddenReflectiveMaterialsNumber = 0	
		@hidden_mats.each {|key, value| 
			mat = Sketchup.active_model.materials[key]
			if (@refl.include?(mat))
				reflectiveMaterialsNumber += 1 unless @mats.has_key?(key)
				hiddenReflectiveMaterialsNumber += 1
				reflectiveMaterials += print_material(key, value.to_i, "hidden_text_button")
			end	
		}
		
		# refracting materials
		refractingMaterials = ""
		refractingMaterialsNumber = 0
		@mats.each {|key, value| 
			mat = Sketchup.active_model.materials[key]
			if (@refra.include?(mat))
				refractingMaterialsNumber += 1
				refractingMaterials += print_material(key, value.to_i)
			end
		}
		hiddenRefractingMaterialsNumber = 0
		@hidden_mats.each {|key, value| 
			mat = Sketchup.active_model.materials[key]
			if (@refra.include?(mat))
				refractingMaterialsNumber += 1 unless @mats.has_key?(key)
				hiddenRefractingMaterialsNumber += 1
				refractingMaterials += print_material(key, value.to_i, "hidden_text_button")
			end
		}
		
		# image unexplode
		unexplodeImages = checkImageExplode
		
		# flipped components/groups
		#flippedEntities = getTransformation

		message += "<br>" + "#{$strh['Materials']}" + ":<br>"
		
		if (@lemMaterialsNumber==1)
			message += "&nbsp;%d " % [@lemMaterialsNumber] + "#{$strh['light-emittingmaterial']}" +  "(" + "#{$strh['LEM']}" + ") <br>"
		else
			message += "&nbsp;%d " % [@lemMaterialsNumber] + "#{$strh['light-emittingmaterials']}" + "(" + "#{$strh['LEM']}" + ") <br>"
		end
		
		if(reflectiveMaterialsNumber==1)
			message += "&nbsp;%d " % [reflectiveMaterialsNumber] + "#{$strh['reflectivematerial']}" + "<br>"
		else
			message += "&nbsp;%d " % [reflectiveMaterialsNumber] + "#{$strh['reflectivematerials']}" + "<br>"
		end
		
		if(refractingMaterialsNumber==1)
			message += "&nbsp;%d " % [refractingMaterialsNumber] + "#{$strh['materialwithrefraction']}" + "<br>"
		else
			message += "&nbsp;%d " % [refractingMaterialsNumber] + "#{$strh['materialswithrefraction']}" + "<br>"
		end
		
		if(@bumpMapMaterialsNumber==1)
			message += "&nbsp;%d " % [@bumpMapMaterialsNumber] + "#{$strh['materialwithbumpmap']}" + "<br>"
		else
			message += "&nbsp;%d " % [@bumpMapMaterialsNumber] + "#{$strh['materialswithbumpmap']}" + "<br>"
		end
		
		message += "<br>" + "#{$strh['Alerts']}" + ":<br>"
		
		if(@msgDTRNumber==1)
			message += "&nbsp;%d " % [@msgDTRNumber] + "#{$strh['DTRproblem']}" + "<br>"
		else
			message += "&nbsp;%d " % [@msgDTRNumber] + "#{$strh['DTRproblems']}" + "<br>"
		end
		
		if(@unexplodedImageNumber==1)
			message += "&nbsp;%d " % [@unexplodedImageNumber] + "#{$strh['unexplodedimage']}" + "<br>"
		else
			message += "&nbsp;%d " % [@unexplodedImageNumber] + "#{$strh['unexplodedimages']}" + "<br>"
		end
		
		if(@msgWrongRenderNumber==1)
			message += "&nbsp;%d psd " % [@msgWrongRenderNumber] + "#{$strh['material']}" + "<br>"
			else
			message += "&nbsp;%d psd " % [@msgWrongRenderNumber] + "#{$strh['materials']}" + "<br>"
		end
		
		message += "&nbsp;%d " % [@noExtensionNumber] + "#{$strh['noextensiontexture']}" + "<br>"
		message += "&nbsp;%d " % [@wrongExtNumber] + "#{$strh['wrongextension']}" + "<br>"
		
		if(@crashMatsNumber==1)
			message += "&nbsp;%d " % [@crashMatsNumber] + "#{$strh['material']}" + " " + "#{$strh['withBothLEMandreflection(cancrash)']}" + "<br>"
		else
			message += "&nbsp;%d " % [@crashMatsNumber] + "#{$strh['materials']}" + " " + "#{$strh['withBothLEMandreflection(cancrash)']}" + "<br>"
		end
		
		#message += "&nbsp;%d flipped component%s/group%s<br>" % [@flippedNumber, @flippedNumber == 1 ? "" : "s", @flippedNumber == 1 ? "" : "s"]
		
		message += "&nbsp;%d " % [@crashHighIntensityLemNumber] + "#{$strh['HighIntensityLEMappliedtomaterials(maycrash)']}" + "<br>"
		
		message += "<br><br>" + "#{$strh['DETAILS']}" + "<br>=================<br>" + "#{$strh['MaterialsDetails']}" + ":" if ((@lemMaterialsNumber > 0) and (lemMaterials != "")) or (reflectiveMaterialsNumber > 0) or (refractingMaterialsNumber > 0) or (@bumpMapMaterialsNumber > 0)
		message += "<br>" + "#{$strh['Light-emittingmaterials']}" +  "(" + $strh['LEM'] + ") <br>" if (@lemMaterialsNumber > 0) and (lemMaterials != "")
		message += lemMaterials
		message += "<br>" + "#{$strh['ReflectiveMaterials']}" + ":<br>" if reflectiveMaterialsNumber > 0
		message += reflectiveMaterials
		message += "<br>" + "#{$strh['Materialswithrefraction']}" + ":<br>" if refractingMaterialsNumber > 0
		message += refractingMaterials
		message += "<br>" + "#{$strh['Materialswithbumpmap']}" + ":<br>" if @bumpMapMaterialsNumber > 0
		message += @bumpMapMaterials
		
		message += "<br>Alerts Details:<br>" if (@msgDTRNumber > 0) or (@msgWrongRenderNumber > 0) or (@unexplodedImageNumber > 0) or (@mats_count > 0) or (@flippedNumber > 0) or (@crashHighIntensityLemNumber > 0)
		
		if @mats_count != 0
		message += "%s " % [@mats_count==1 ? "#{$strh['Thismaterial']}" : "#{$strh['Thesematerials']}"] + "#{$strh['willnotrender']}" + ":<br>"
		message += "&nbsp;&nbsp;" + "#{$strh['Noextensiontexture']}" + ": <br>%s" % [@mats_name] if @mats_name.length > 0
		message += "&nbsp;&nbsp;" + "#{$strh['Addextensionstoallmaterials?']} <button onclick='call_ruby(\"fixNoExtention\")' class=\"hidden_text_button\">#{$strh['Click']} #{$strh['here']}</button>" if @mats_name.length > 0
		message += "&nbsp;&nbsp;" + "#{$strh['Wrongformattexture']}" + ": <br>%s" % [@mats_ext_wrong] if @mats_ext_wrong.length > 0
		message += "&nbsp;&nbsp;" + "#{$strh['BothLEMandreflections(cancrash)']}" + ": <br>%s" % [@mats_crash] if @mats_crash.length > 0
		end

		# DTR details
		message += "<br>" + @msgDTR if @msgDTRNumber > 0

		# psd files details
		message += "<br>" + "#{$strh['Unknownsymbol']}" + ":" if @msgWrongRender.length > 0
		message += @msgWrongRender if @msgWrongRender.length > 0

		# unexploded images details
		message += unexplodeImages if @unexplodedImageNumber > 0
		
		#flipped components
		#message += "<br>Fliped component%s/Group%s:<br>" % [@flippedNumber == 1 ? "" : "s", @flippedNumber == 1 ? "" : "s"] if @flippedNumber > 0
		#message += flippedEntities if @flippedNumber > 0
		
		if(@crashHighIntensityLemNumber == 1)
			message += "<br>" + "#{$strh['HighintensityLEM']}" + "#{$strh['texture']}" + ":<br>" if @crashHighIntensityLemNumber > 0
		else
			message += "<br>" + "#{$strh['HighintensityLEM']}" + "#{$strh['textures']}" + ":<br>" if @crashHighIntensityLemNumber > 0
		end
		
		message += @highIntensityLem if @crashHighIntensityLemNumber > 0
		message += "<br> " + "#{$strh['Click']}" + " <a href=\"http://www.suplugins.com/help2/other.php#tools\" target=\"_blank\">" + "#{$strh['here']}" + "</a> " + "#{$strh['fortoreadaboutAnalyseTools']}" + "." if (@msgDTRNumber > 0) or (@msgWrongRenderNumber > 0) or (@unexplodedImageNumber > 0) or (@mats_count > 0)
		
		model = Sketchup.active_model
		number_of_faces = model.number_faces
		number_of_materials = model.materials.count
		number_of_layers = model.layers.count
		number_of_pages = model.pages.count
		
		osInfo = ""
		# get Os version
		if @m_MAC # You are running on a Mac computer.
			mac_version_path = File.expand_path('~')
			mac_version_path = mac_version_path + "/macVersion.txt"
			# call cmd
			`sw_vers > #{mac_version_path}`
			#read file
			if(File.exist?(mac_version_path))
				osInfo = File.read(mac_version_path)
				osInfo = "MAC (" + osInfo + ")"
				#delete file
				FileUtils.rm mac_version_path
			else
				osInfo = "MAC (Unknown Version)"
			end
		elsif @m_WIN # You are running on a Windows computer.
			windows_version_path = File.expand_path('~')
			windows_version_path = windows_version_path + "/windowsVersion.txt"
			# call cmd
			`ver > #{windows_version_path}`
			#read file
			if(File.exist?(windows_version_path))
				read_file = File.read(windows_version_path)
				#delete file
				FileUtils.rm windows_version_path
				# get version from string
				version = read_file.scan(/\d+/)
				osInfo = findWindowsVersionName(version[0], version[1])
			else
				osInfo = "Windows (Unknown version)"
			end
		else # You are running on another architecture.
			osInfo = "Unknown"
		end	
		
		message += "<br><br>" + "#{$strh['GENERALMODELINFO']}" + "<br>==================================<br>"
		message += "<br>" + "#{$strh['OperatingSystem']}" + ": #{osInfo}"
		message += "<br>" + "#{$strh['SketchupVersion']}" + ": %s" % [Sketchup.version.to_s]
		message += "<br>" + "#{$strh['SUPodiumVersion']}" + ": %s" % [Podium::VERSION] if (Object.const_defined?('Podium') == true)
		message += "<br>" + "#{$strh['Thereare']}" + " #{number_of_faces} %s, #{number_of_materials} %s, #{number_of_pages} %s, #{number_of_layers}  %s, #{@number_of_Groups} %s " % [number_of_faces == 1 ? "#{$strh['face']}" : "#{$strh['faces']}", number_of_materials == 1 ? "#{$strh['material']}" : "#{$strh['materials']}", number_of_pages == 1 ? "#{$strh['page']}" : "#{$strh['pages']}", number_of_layers == 1 ? "#{$strh['layer']}" : "#{$strh['layers']}", @number_of_Groups == 1 ? "#{$strh['group']}" : "#{$strh['groups']}"] + "#{$strh['and']}" + " #{@number_of_ComponentInstances} %s "  % [@number_of_ComponentInstances == 1 ? "#{$strh['componentinstance']}" : "#{$strh['componentinstances']}"] + "#{$strh['inthemodel']}"  + "."
															
		message += "<br><br>&nbsp;&nbsp;<button onclick='call_ruby(\"save\", 0)' >" + "#{$strh['SaveResults']}" + "</button> <br> <br>"
		podium_image_folder_path =
			Sketchup.find_support_file( 'Plugins/SU_Podium_V2/plug_in/views/html/images' )
		arrow_icon =
			File.join( podium_image_folder_path, 'arrow.bmp' ) unless  podium_image_folder_path.nil?

		# show message
		html = <<-END_HERE
		<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
		<html>
			<head>
				<meta charset="UTF-8">
			</head>
			<body bgcolor = "F0F0F0">
				<title>SU Podium Analysis Results</title>
				<script>
					function call_ruby( callback_name, message ) {
						location = 'skp:' + callback_name + '@' + message
					}
				</script>
				<style>
					.text_button {

						overflow:visible; 
						font-size:1em;
					border: none;
					background-color: transparent;
					padding: 0;
					text-decoration: none; /* if desired */
					color: #0000FF;  /* or whatever other color you want */
					}
					.hidden_text_button {

						overflow:visible; 
						font-size:1em;
					border: none;
					background-color: transparent;
					padding: 0;
					text-decoration: none; /* if desired */
					color: #A10000;  /* or whatever other color you want */
					}
				</style>
		END_HERE

		message = html + message + "</body></html>"
		
		# save Analyze
		@save_analysis = message
		@@webDialog.add_action_callback( "save" ) do |dlg, message|
			save_analyze(@save_analysis)
		end
		
		# select materials
		@@webDialog.add_action_callback( "material" ) do |dlg, message|
			select(message)
		end
		
		# select and explode images
		@@webDialog.add_action_callback( "explodeByID" ) do |dlg, message|
			find_and_explode(message)
		end
		
		# select and explode images
		@@webDialog.add_action_callback( "fixNoExtention" ) do |dlg, message|
			fix_No_Extention(message)
		end
		
		# select selectById components
		@@webDialog.add_action_callback( "selectById" ) do |dlg, message|
			model = Sketchup.active_model
			selection = model.selection
			
			#find entity
			ent = model.find_entity_by_id(message.to_i)
			
			# clear selection
			selection.clear
			
			# add entity to selection
			selection.add ent
		end
		
		@@webDialog.set_html(message)
		
		@@webDialog.show()
		
		return message
	end
  
	def save_analyze(message)
		time = Time.new
		model = Sketchup.active_model
		path = File.dirname model.path
		extn = File.extname  model.path
		currentTime = "#{time.day}_#{time.month}_#{time.year}_#{time.hour}_#{time.min}_#{time.sec}"
		
		model_name = File.basename model.path, extn
		full_path = ""
		
		if @m_WIN
			full_path = "#{path}\\#{model_name}_#{currentTime}.html"
		else
			full_path = "#{path}/#{model_name}_#{currentTime}.html"
		end
		
		File.open(full_path, 'w') do |file|  
		  file.puts message
		end  
		#system("start \"#{path}\"")
		UI.messagebox("Results saved successfully in location:\n\n#{full_path}")
	end
	
	def fix_No_Extention(name)
		model = Sketchup.active_model
        begin
			current = 0
			percent = -1
            op_name = "Fix 'no extension' materials"
            model.start_operation( op_name )
                model = Sketchup.active_model
                materials  = model.materials
                textures_path = Sketchup.find_support_file('materials') + "//"
                mat_count = materials.count

				materials.each do |x|
					# progress bar
					new_percent = ((current * 100)/mat_count).to_i
					if (new_percent != percent)
						status "#{$strh['Fixingnoextension']}: %2d%%" % new_percent
						percent = new_percent
					end
					current += 1
			
					next if x.texture.nil?
					ext = /(.*)\.(.*$)/.match(x.texture.filename)
					ext = ext[2] unless ext.nil?
					if ext.nil?
						@no_extension_materials = true
						old_x = x.texture.width
						old_y = x.texture.height
						x.texture.write(textures_path + x.name + "-texture.jpg")
						x.texture = textures_path + x.name + "-texture.jpg"
						x.texture.size = [old_x, old_y]
					end
                end
                
                if @no_extension_materials
                    UI.messagebox("#{$strh['Fixednoextensionmaterials']}")
                else
                    UI.messagebox("#{$strh['Thereisnothingtodo.Nomaterialswithnoextensionfound.']}")
                end
				status "#{$strh['Done']}"
            model.commit_operation
            rescue => e
            model.abort_operation
            puts("Error #<#{e.class.name}:#{e.message}.>")
            puts(e.backtrace) if $VERBOSE
       
        else
        # Do code here ONLY if NO errors occur.
        ensure
        # ALWAYS do code here errors or not.
        end
	end
	
	def find_and_explode(name)
		model = Sketchup.active_model
		selection = model.selection
		
		#find entity
		ent = model.find_entity_by_id(name.to_i)

		ent.explode
		
		# display message
		result = UI.messagebox("#{$strh['Explodesuccess']}!", MB_OK)
	end
	
	def select(name)
		model = Sketchup.active_model
		materials = model.materials
		
		materials.each do |material|
			if material.name.eql?(name)		
				#Podium::MaterialsController.new.show_materials_browser if((defined? Podium) != nil)
				materials.current = material
				return material
				break
			end
		end
	end
	
	def material_check
		current = 1
		percent = -1
		
		mats = Sketchup.active_model.materials
		mat_count = Sketchup.active_model.materials.count
		dtrPercentage = 50
		@msgDTR = "#{$strh['DTRproblems']}:<br>"
		mats.each do |x|
			# Count reflective and LEM materials
			dict = x.attribute_dictionaries
			dtr = 0
			
			new_percent = ((current * 100)/mat_count).to_i
			if (new_percent != percent)
				status "#{$strh['AnalyzingMaterials']} %2d%%" % new_percent
				percent = new_percent
			end
			current += 1
	  
			if (dict)
				dict = dict["podium_II"]	
			end
			
			if (dict and (@mats.has_key?(x.name) or @hidden_mats.has_key?(x.name)))
				is_lem = false
				is_refl = false
				is_refra = false
		
				if (dict["bump_depth"].to_i > 0)
					@bumpMapMaterialsNumber += 1
					
					if @hidden_mats.has_key?(x.name)
						@hiddenBumpMapMaterialsNumber += 1
						@bumpMapMaterials += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >\"#{x.name}\"</button><br>" 					
					else
						@bumpMapMaterials += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >\"#{x.name}\"</button><br>" 
					end	
				end
				if (dict["light_power"].to_i > 0)
					is_lem = true
					@LEM << x
					@lemMaterialsNumber += 1
					if (dict["hidden_lem"] == true)
						@hiddenLEMNumber += 1
					elsif ((dict["light_power_high_intensity"] == true))
						@highIntesityLemNumber += 1
						#crash HI LEM
						if (!x.texture.nil?)
							@crashHighIntensityLemNumber += 1
							if @hidden_mats.has_key?(x.name)
								@crashHiddenHighIntensityLemNumber += 1
								@highIntensityLem += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >\"#{x.name}\"</button><br>" 					
							else
								@highIntensityLem += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >\"#{x.name}\"</button><br>" 
							end
						end
					else	
						@normalLemMaterialsNumber += 1
					end
				end
				if (dict["reflection"].to_f > 0.0)
					is_refl = true
					@refl << x
				end
				if (dict["refraction"].to_f > 0.0)
					is_refra = true
					@refra << x
				end
				if (dict["refraction"].to_f > 0.0)
					is_refra = true
					@refra << x
				end
				if (is_lem && is_refl)
					@mats_count += 1
					@crashMatsNumber += 1
					if @hidden_mats.has_key?(x.name)
						@hiddenCrashMatsNumber += 1
						@mats_crash += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >\"#{x.name}\"</button><br>" 					
					else
						@mats_crash += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >\"#{x.name}\"</button><br>" 
					end
				end
				
				dtr = dict["diffuse"].to_f + (1 - (x.alpha).to_f) + dict["reflection"].to_f
				if ((dtr*100) < dtrPercentage)
					@msgDTRNumber += 1
					if @hidden_mats.has_key?(x.name)
						@hiddenMsgDTRNumber += 1
						@msgDTR += "&nbsp;&nbsp;    \"<button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >#{x.name}</button>\" has less than #{dtrPercentage}% DTR value.<br>"
					else
						@msgDTR += "&nbsp;&nbsp;    \"<button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >#{x.name}</button>\" has less than #{dtrPercentage}% DTR value.<br>"
					end
				end 		
			end
		end		
	end
	
	def wrongRender_unkownSymbol_check 
		arrWrongRenderExt = Array.new()
		
		model = Sketchup.active_model
		materials  = model.materials
		entities = model.entities
		current = 1
		percent = -1
		ent_count = entities.count
		mat_count = materials.count
		count = ent_count + mat_count
		materials.each do |x|
			new_percent = ((current * 100)/count).to_i
			if (new_percent != percent)
				status "#{$strh['AnalyzingWrongRenderandUnknownSymbol']} %2d%%" % new_percent
				percent = new_percent
			end
			current += 1
			next if x.texture.nil?
			ext = /(.*)\.(.*$)/.match(x.texture.filename)
			ext = ext[2] unless ext.nil?
			if ext.nil?
				ext = "no extension" 
				@mats_count += 1
				@noExtensionNumber += 1
				if @hidden_mats.has_key?(x.name)
					@hiddenNoExtensionNumber += 1
					@mats_name += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >\"#{x.name}.#{ext}\"</button>&nbsp;<br>" 
				else
					@mats_name += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >\"#{x.name}.#{ext}\"</button>&nbsp;<br>"
				end
			elsif ( !@supported_textures.include?( ext.downcase ) )
				@mats_count += 1
				@wrongExtNumber += 1
				if @hidden_mats.has_key?(x.name)
					@hiddenWrongExtNumber += 1
					@mats_ext_wrong += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >\"#{x.name}.#{ext}\"</button><br>" 
				else
					@mats_ext_wrong += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >\"#{x.name}.#{ext}\"</button><br>" 
				end
			end
			ext = /(.*)\.(.*$)/.match(x.texture.filename)
			ext = ext[2] unless ext.nil?
			# check for .psd
			if not ext.nil?
				if ( ext.downcase.include?("psd") )
					@msgWrongRenderNumber += 1
					if @hidden_mats.has_key?(x.name)
						@hiddenMsgWrongRenderNumber += 1
						temp = "&nbsp;&nbsp;  <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"hidden_text_button\" >\"#{x.name}.#{ext}\"</button><br>"
					else
						temp = "&nbsp;&nbsp;  <button onclick='call_ruby(\"material\", \"#{x.name}\")' class=\"text_button\" >\"#{x.name}.#{ext}\"</button><br>"
					end
					arrWrongRenderExt << temp.to_s
				end
			end
		end
		
		entities.each do |x|
			new_percent = ((current * 100)/count).to_i
			if (new_percent != percent)
				status "#{$strh['AnalyzingWrongRenderandUnknownSymbol']} %2d%%" % new_percent
				percent = new_percent
			end
			current += 1
			case x.typename
			when "Image"
				ext = /(.*)\.(.*$)/.match(x.path)[2]
				name = File.basename x.path
				if ext.nil?
					ext = "no extension" 
					@mats_count += 1
					@noExtensionNumber += 1
					if ent.hidden?
						@hiddenNoExtensionNumber += 1
						@mats_name += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"selectById\", \"#{x.entityID}\")' class=\"hidden_text_button\" >\"#{name}\"</button><br>" 
					else
						@mats_name += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"selectById\", \"#{x.entityID}\")' class=\"text_button\" >#{name}</button><br>" 
					end
				elsif ( !@supported_textures.include?( ext.downcase ) )
					@mats_count += 1
					@wrongExtNumber += 1
					if x.hidden?
						@hiddenWrongExtNumber += 1
						@mats_ext_wrong += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"selectById\", \"#{x.entityID}\")' class=\"hidden_text_button\" >\"#{name}\"</button> <br>" 
					else
						@mats_ext_wrong += "&nbsp;&nbsp;&nbsp;      <button onclick='call_ruby(\"selectById\", \"#{x.entityID}\")' class=\"text_button\" >\"#{name}\"</button> <br>" 
					end
				end
				ext = /(.*)\.(.*$)/.match(x.path)[2]
				# check for .psd
				if ( ext.downcase.include?("psd") )
					@msgWrongRenderNumber += 1	
					name = File.basename x.path
					
					if x.hidden?
						@hiddenMsgWrongRenderNumber += 1
						temp = "&nbsp;&nbsp;  <button onclick='call_ruby(\"selectById\", \"#{x.entityID}\")' class=\"hidden_text_button\" >\"#{name}\"</button><br>"
					else	
						temp = "&nbsp;&nbsp;  <button onclick='call_ruby(\"selectById\", \"#{x.entityID}\")' class=\"text_button\" >\"#{name}\"</button><br>"
					end
					arrWrongRenderExt << temp.to_s
				end
			end
		end
		if(arrWrongRenderExt.length > 0)	
			@msgWrongRender += "<br>" + arrWrongRenderExt.uniq.join() + ".psd #{$strh['materialswillcauseOOPRtodisplayanunknownsymbolmessage']}.<br><br>"
		end	
	end
	
	def checkImageExplode()
		msgImg = "#{$strh['Unexplodedimages']}:<br>"
		Sketchup.active_model.entities.each do |ent| 
			#Image not explode
			case ent.typename
			when "ComponentInstance"
				@number_of_ComponentInstances += 1
				# add here suplimentary code if it will be necessary to check explode
				#puts("ComponentInstance")
				#entities = ent.definition.entities
				#puts(entities)
			when "Image"#means image is not explode
				ext = /(.*)\.(.*$)/.match(ent.path)[2]
				name = File.basename ent.path
				@unexplodedImageNumber += 1	
				
				if ent.hidden?
					@hiddenUnexplodedImageNumber += 1
					msgImg += " &nbsp;&nbsp;  	<button onclick='call_ruby(\"selectById\", \"#{ent.entityID}\")' class=\"hidden_text_button\" >\"#{name}\"</button>&nbsp;&nbsp;<button onclick='call_ruby(\"explodeByID\", \"#{ent.entityID}\")' >#{$strh['Explode']}</button><br>"
				else
					msgImg += " &nbsp;&nbsp;  	<button onclick='call_ruby(\"selectById\", \"#{ent.entityID}\")' class=\"text_button\" >\"#{name}\"</button>&nbsp;&nbsp;<button onclick='call_ruby(\"explodeByID\", \"#{ent.entityID}\")' >#{$strh['Explode']}</button><br>"
				end					
			when "Group"
				@number_of_Groups += 1
				# add here suplimentary code if it will be necessary to check explode	
				#puts("Group")
				#entities = ent.entities
				#puts(entities)
			end
		end
		#msgImg += "<br> #{$strh['Click']} <a href=\"http://www.suplugins.com/help-faq-v2-2.php#bugs\">#{$strh['here']}</a> #{$strh['formoreinformation']}." if msgImg.length > 0
		return msgImg
	end
		
	# Returns the windows version  
	def getWindowsVersion  
		require "Win32API"
		getVersionEx = Win32API.new('kernel32','GetVersionEx','P','L')  
		
		osversioninfo = [
		148,		# size of this struct (IN)
		0,		# major version (OUT)
		0,		# minor version (OUT)
		0,		# build (OUT)
		0,		# platform (OUT)
		"\0" * 128	# additional info (OUT)
		].pack('LLLLLa128')
		
		getVersionEx.call(osversioninfo)	# => 1, means succeeded	
		return osversioninfo.unpack('LLLLLZ128')# 'Z' means ASCIIZ string
	end 
		
	def findWindowsVersionName(major, minor)
		case "#{major}.#{minor}".to_f
			when 5.0
				return "Windows 2000"
			when 5.1
				return "Windows XP"
			when 5.2
				return "Windows Server 2003 #{$strh['or']} Windows Server 2003 R2"
			when 6.0
				return "Windows Vista #{$strh['or']} Windows Server 2008"
			when 6.1
				return "Windows 7 #{$strh['or']} Windows Server 2008 R2"
			when 6.2
				return "Windows 8 #{$strh['or']} Windows Server 2012"
			when 6.3
				return "Windows 8.1 #{$strh['or']} Windows Server 2012 R2"
			when 10.0
				return "Windows 10"
		end	
	end

	def print_material(name, count, button_class = "text_button")
		name = name.sub("<", "&lt;")
		name = name.sub(">", "&gt;")
		
		return "&nbsp;&nbsp;    \"<button onclick='call_ruby(\"material\", \"%s\")' class=\"%s\" >%s</button>\" #{$strh['usedin']} %d %s<br>" % [name, button_class, name, count, count==1 ? "#{$strh['face']}" : "#{$strh['faces']}"]
	end

	def do_test_reset_materials
		mats = Sketchup.active_model.materials
		mats.each do |x|
			dict = x.attribute_dictionaries
			if (dict !=nil && dict["podium_II"] != nil)
			dict.delete("podium_II")
			end	 
		end
	end

	def reset_materials
		ResetMaterialsDialog.new
	end

	def reset_selected_materials(refraction, lem, blur, bump, hiddenLEM)
		nchanges = 0
		mats = Sketchup.active_model.materials
		mats.each do |x|
			dict = x.attribute_dictionaries
			if (dict)
				dict = dict["podium_II"]
			end
			if (dict)
				if (lem && dict["light_power"].to_i > 0)
					dict["light_power"] = 0
					nchanges+=1
				end
				if (hiddenLEM && dict["hidden_lem"] == true)
					dict["light_power"] = 0
					nchanges+=1
				end
				if (blur)
					if (dict["reflection_blur"])
						dict["reflection_blur"] = false
						nchanges+=1
					end
					if (dict["refraction_blur"])
						dict["refraction_blur"] = false
						nchanges+=1
					end
				end
				if (bump)
					if (dict["bump_depth"])
						dict["bump_depth"] = 0
						nchanges+=1
					end
				end
				if (refraction)
					if (dict["refraction"])
						dict["refraction"] = 0
						nchanges+=1
					end
				end
			end
		end    
		puts nchanges.to_s + " changes."
		go
	end

	def test_reset_materials_confirm
		message = "#{$strh['ThiswillremoveallSUPodiummaterialproperties']}.\n"
		message += "#{$strh['Areyousureyouwanttocontinue?']}"
		save = UI::messagebox(message, MB_YESNO);
		return if(save==IDNO)
		do_test_reset_materials
		status "#{$strh['SUPodiummaterialinformationdeleted']}."
		go
	end

	def do_reset_model
		do_test_reset_materials
		ent_count = Sketchup.active_model.entities.count
		Sketchup.active_model.entities.each do |x| 
		remove_ent(x)
		end
	end

	def reset_model
		message = "#{$strh['ThiswillremoveallSUPodiumproperties']}."
		message += "<br><br> #{$strh['Areyousureyouwanttocontinue?']}"
		save = UI::messagebox(message, MB_YESNO);
		return if(save==IDNO)
		do_reset_model
		status "#{$strh['SUPodiuminformationinmodeldeleted']}."
		go
	end

	def remove_ent(ent)
		dict = ent.attribute_dictionaries

		if (dict !=nil && dict["podium_II"] != nil)
		dict.delete("podium_II")
		end	 
	
		case ent.typename
		when "ComponentInstance"
			remove_ent(ent.definition)
			entities = ent.definition.entities
		when "Group"	
			entities = ent.entities
		end
		
		if entities
			entities.each do |subEntity| 
				remove_ent(subEntity)
			end
		end
	end

	def proc_face(ent)
		#return if ent.hidden?
		case ent.typename
		when "Face"
			mat = ent.material
			bmat = ent.back_material
			if (mat)	
				if ent.hidden?
					@hidden_mats[mat.name] = @hidden_mats[mat.name] + 1
				else
					@mats[mat.name] = @mats[mat.name] + 1
				end
			end
			if (bmat && bmat != mat) 		
				if ent.hidden?
					@hidden_mats[bmat.name] = @hidden_mats[bmat.name] + 1
				else
					@mats[bmat.name] = @mats[bmat.name] + 1
				end
			end
		when "ComponentInstance"
			entities = ent.definition.entities
		when "Group"	
			entities = ent.entities
		end
		
		if entities
		entities.each do |subEntity| 
			proc_face(subEntity)
		end
		end
	end  

	def faces_check
		ent_count = Sketchup.active_model.entities.count
		current = 1
		percent = -1
		Sketchup.active_model.entities.each do |x| 
		proc_face(x)
		new_percent = ((current * 100)/ent_count).to_i
		if (new_percent != percent)
			status "#{$strh['AnalyzingFaces']} %2d%%" % new_percent
			percent = new_percent
		end
		current += 1
		end
	end

	def entities_check
		ent_count = Sketchup.active_model.entities.count
		current = 1
		percent = -1
		Sketchup.active_model.entities.each do |x| 
		proc_ent(x)
		new_percent = ((current * 100)/ent_count).to_i
		if (new_percent != percent)
			status "#{$strh['AnalyzingEntities']} %2d%%" % new_percent
			percent = new_percent
		end
		current += 1
		end
	end

	def automatic_materials
		# Read names out of the podium materials file
		file_name = Sketchup.find_support_file("materials.csv", "Plugins/SU_Podium_V2/materials")
		if (! file_name)
			puts("No podium materials file")
			return
		end
		
		readfile = File.open(file_name, "r")
		if (! readfile)
			puts("No podium materials")
			return
		end
		
		while line = readfile.gets
			first = line.index("[")
			last  = line.index("]")
			if (first >= 0 and last > 0 and (last - first) > 1)
				mname = line[first+1, last-first-1]
				@podium_materials << mname
			end
		end
		readfile.close
	end

	def status(msg)
		Sketchup.set_status_text "#{$strh['PodiumTools']} :: #{msg}"
	end

	def mark(ent,text)
		Sketchup.active_model.selection.add ent
	end
	
	def move_to(ent, layer)
		ent.layer = layer	  
	end
	
	def proc_light(ent,power)
		return if power == 0.0
		@light_count += 1
		@lem = Sketchup.active_model.layers.add "PDM_LEMs" if @lem == nil
		move_to(ent, @lem)
	end
	
	def proc_omni(ent,power)
		return if power == 0.0  
		@omni_count += 1
		@omni = Sketchup.active_model.layers.add "PDM_Omnis" if @omni == nil	
		move_to(ent, @omni)
	end
	
	def proc_instance(ent,power)
		# An instance is a light if it's "light-type" is "point"
		# attribute and it's light-power, if any, is non-zero.
		#return if power == 0.0  
		dict = ent.definition.attribute_dictionaries
		if (dict)
			dict = ent.definition.attribute_dictionaries["podium_II"]
		end
		if (dict)
			if (dict["light-type"] == "point")
				@light_count += 1
				@off_light_count += 1 if power == 0.0 
				if ent.hidden?
					@hidden_light_count += 1
				end
			elsif (dict["light-type"] == "softspot")
				@spot_count += 1
				@off_spot_count += 1 if power == 0.0 
				if ent.hidden?
					@hidden_spot_count += 1
				end
			end
		end
	end

	def proc_ent(ent)
		return if ent.hidden?
		dict = ent.attribute_dictionaries

		if (dict !=nil && dict["podium_II"] != nil)
			power = 1
		dict["podium_II"].each_pair do |k,v|
			case k
			when "light-power"
				power = v
			when "OmniLightP"
				proc_omni(ent,v)
			end
		end
		end	 
	
		case ent.typename
		when "ComponentInstance"
			proc_instance(ent, power)
			entities = ent.definition.entities
		when "Group"	
			entities = ent.entities
		end
		
		if entities
			entities.each do |subEntity| 
				proc_ent(subEntity)
			end
		end
	end  

	def count_point_lights_instance(ent)
		dict = ent.definition.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-type"] == "point")
			# This is a light
		else
			return
		end

		power = 1.0
		dict = ent.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-power"])
			power = dict["light-power"]
		end

		if (power.to_f == 0.0)
			@off += 1
		else
			@on += 1
		end
	end  

	def count_point_lights_ent(ent)
		return if ent.hidden?
		case ent.typename
		when "ComponentInstance"
			count_point_lights_instance(ent)
			entities = ent.definition.entities
		when "Group"	
			entities = ent.entities
		end
		
		if entities
			entities.each do |subEntity| 
				count_point_lights_ent(subEntity)
			end
		end
	end  

	def count_point_lights
		ent_count = Sketchup.active_model.entities.count
		Sketchup.active_model.entities.each do |x| 
			count_point_lights_ent(x)
		end
	end  

	def test_toggle_point_lights_instance(ent, off)
		dict = ent.definition.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-type"] == "point")
			# This is a light
		else
			return
		end

		power = 1.0
		dict = ent.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-power"])
			power = dict["light-power"]
		end

		# if turning off, save the power value, if any, and set it to zero
		if (off)
			return if (power.to_f == 0.0)
			ent.set_attribute("podium_II", "original-light-power", power)
			ent.set_attribute("podium_II", "light-power", 0.0)
		else
			return if (power.to_f != 0.0)
			power = ent.get_attribute("podium_II", "original-light-power", 1.0)
			ent.set_attribute("podium_II", "light-power", power)
		end
	end  

	def test_toggle_point_lights_ent(ent, off)
		return if ent.hidden?
		case ent.typename
		when "ComponentInstance"
			test_toggle_point_lights_instance(ent, off)
			entities = ent.definition.entities
		when "Group"	
			entities = ent.entities
		end
		
		if entities
			entities.each do |subEntity| 
				test_toggle_point_lights_ent(subEntity, off)
			end
		end
	end  

	def do_test_toggle_point_lights(off)
		ent_count = Sketchup.active_model.entities.count
		Sketchup.active_model.entities.each do |x| 
			test_toggle_point_lights_ent(x, off)
		end
	end  

	def toggle_point_lights
		@on = 0.to_i
		@off = 0.to_i
		count_point_lights
		if (@on == 0 && @off == 0)
			UI.messagebox("#{$strh['Nopointlightsinthismodel']}.")
			return
		elsif (@on > @off)
			message = "%d #{$strh['of']} %d #{$strh['pointlightsareon']}. #{$strh['Turnthemoff?']}" % [@on, @on+@off]
		else
			message = "%d #{$strh['of']} %d #{$strh['pointlightsareoff']}. #{$strh['Turnthemon?']}" % [@off, @on+@off]
		end
		result = UI.messagebox(message, MB_YESNO)
		return if (result==IDNO)

		do_test_toggle_point_lights(@on > @off)
		go
	end

	def count_spot_lights_instance(ent)
		dict = ent.definition.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-type"] == "softspot")
			# This is a light
		else
			return
		end

		power = 1.0
		dict = ent.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-power"])
			power = dict["light-power"]
		end

		if (power.to_f == 0.0)
			@off += 1
		else
			@on += 1
		end
	end  

	def count_spot_lights_ent(ent)
		return if ent.hidden?
		case ent.typename
		when "ComponentInstance"
			count_spot_lights_instance(ent)
			entities = ent.definition.entities
		when "Group"	
			entities = ent.entities
		end
		
		if entities
			entities.each do |subEntity| 
				count_spot_lights_ent(subEntity)
			end
		end
	end  

	def count_spot_lights
		ent_count = Sketchup.active_model.entities.count
		Sketchup.active_model.entities.each do |x| 
		count_spot_lights_ent(x)
		end
	end

	def test_toggle_spot_lights_instance(ent, off)
		dict = ent.definition.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-type"] == "softspot")
			# This is a light
		else
			return
		end

		power = 1.0
		dict = ent.attribute_dictionaries
		if (dict)
			dict = dict["podium_II"]
		end
		if (dict && dict["light-power"])
			power = dict["light-power"]
		end

		# if turning off, save the power value, if any, and set it to zero
		if (off)
			return if (power.to_f == 0.0)
			ent.set_attribute("podium_II", "original-light-power", power)
			ent.set_attribute("podium_II", "light-power", 0.0)
		else
			return if (power.to_f != 0.0)
			power = ent.get_attribute("podium_II", "original-light-power", 1.0)
			ent.set_attribute("podium_II", "light-power", power)
		end
	end  

	def test_toggle_spot_lights_ent(ent, off)
		return if ent.hidden?
		case ent.typename
		  when "ComponentInstance"
			test_toggle_spot_lights_instance(ent, off)
			entities = ent.definition.entities
		  when "Group"	
			entities = ent.entities
		end

		if entities
		  entities.each do |subEntity| 
			test_toggle_spot_lights_ent(subEntity, off)
		  end
		end
	end  

	def do_test_toggle_spot_lights(off)
		ent_count = Sketchup.active_model.entities.count
		Sketchup.active_model.entities.each do |x| 
		  test_toggle_spot_lights_ent(x, off)
		end
	end  

	def toggle_spot_lights
		@on = 0.to_i
		@off = 0.to_i
		count_spot_lights
		if (@on == 0 && @off == 0)
			UI.messagebox("#{$strh['Nospotlightsinthismodel']}.")
			return
		elsif (@on > @off)
			message = "%d #{$strh['of']} %d #{$strh['spotlightsareon']}. #{$strh['Turnthemoff?']}" % [@on, @on+@off]
		else
			message = "%d #{$strh['of']} %d #{$strh['spotlightsareoff']}. #{$strh['Turnthemon?']}" % [@off, @on+@off]
		end
		result = UI.messagebox(message, MB_YESNO)
		return if (result==IDNO)

		do_test_toggle_spot_lights(@on > @off)
		go
	end
	
	def getTransformation()
		message = ""
		flipedComponentsString = ""
		unnamedGroupIndex = 0
		unnamedComponentIndex = 0
		
		Sketchup.active_model.entities.each do |ent| 	
		case ent.typename
			# Face,Edge can't be fliped
			when "ComponentInstance", "Group"
				# get transfromation
				axisInfo = ent.transformation
				if ((axisInfo.xaxis[0] < 0) || (axisInfo.yaxis[1] < 0) || (axisInfo.zaxis[2] < 0))				
					if (ent.name.empty? or (ent.name == ""))
						name = "Unnamed %s_%d" % [ent.typename, (ent.typename == "Group") ? (unnamedGroupIndex += 1) : (unnamedComponentIndex += 1)]
					else
						name = ent.name
					end
					
					if ent.hidden?
						flipedComponentsString += "\"<button onclick='call_ruby(\"selectById\", \"#{ent.entityID}\")' class=\"hidden_text_button\" >#{name}</button>\"<br>"
						@hiddenFlippedNumber += 1
					else
						flipedComponentsString += "\"<button onclick='call_ruby(\"selectById\", \"#{ent.entityID}\")' class=\"text_button\" >#{name}</button>\"<br>"
					end
					@flippedNumber += 1
				end
			end
		end
			 
		if @flippedNumber > 0 
			message += flipedComponentsString 
		end
		return message
	end
end

class ResetMaterialsDialog
	#@@resetMaterialsWindow = UI::WebDialog.new( 'Reset Materials', false, 'Reset_Materials', 280, 300, 700, 500, false )
	
	@@dialog = UI::HtmlDialog.new({
		:dialog_title => "Reset Materials",
		:preferences_key => "Reset_Materials",
		:scrollable => false,
		:resizable => false,
		:width => 300,
		:height => 300,
		:left => 100,
		:top => 100,
		:min_width => 50,
		:min_height => 50,
		:max_width =>1000,
		:max_height => 1000,
		:style => UI::HtmlDialog::STYLE_DIALOG
	})
	def initialize()
		@htmlString = <<-htmlString
		<!DOCTYPE html>
		<html lang="en">
			<head>
				<title>Reset Materials</title>
				<meta charset="utf-8">
				<meta name="viewport" content="width=device-width, initial-scale=1">
				<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/css/bootstrap.min.css" integrity="sha384-Gn5384xqQ1aoWXA+058RXPxPg6fy4IWvTNh0E263XmFcJlSAwiGgFAW/dAiS6JXm" crossorigin="anonymous">
				<script src="https://code.jquery.com/jquery-3.2.1.slim.min.js" integrity="sha384-KJ3o2DKtIkvYIK3UENzmM7KCkRr/rE9/Qpg6aAZGJwFDMVNA/GpGFF93hXpG5KkN" crossorigin="anonymous"></script>
				<script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.12.9/umd/popper.min.js" integrity="sha384-ApNbgh9B+Y1QKtv3Rn7W3mgPxhU9K/ScQsAP7hUibX39j7fakFPskvXusvfa0b4Q" crossorigin="anonymous"></script>
				<script src="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/js/bootstrap.min.js" integrity="sha384-JZR6Spejh4U02d8jOt6vLEHfe/JQGiRRSQQxSfFWpi1MquVdAyjUar5+76PVCmYl" crossorigin="anonymous"></script>
				<script>
					function call_ruby( callback_name, message ) {
						location = 'skp:' + callback_name
					}
					
					function call_ruby1( callback_name ) {
						var values = [0,0,0,0,0,0];
						
						var x = document.getElementById("rmPP");
						values[0] = x.checked;
						var x = document.getElementById("rmLEM");
						values[1] = x.checked;
						var x = document.getElementById("rmBur");
						values[2] = x.checked;
						var x = document.getElementById("rmBump");
						values[3] = x.checked;
						var x = document.getElementById("rmHiddenLEM");
						values[4] = x.checked;
						var x = document.getElementById("rmAll");
						values[5] = x.checked;
						
						location = 'skp:' + callback_name + '@' + values
					}
				</script>
				<style>
				body {
					background-color: #f3f3f7;
					font-family: "Open Sans", "Helvetica Neue", Arial, Helvetica, sans-serif,
						Meiryo;
					font-size: 12px;
				}
			</style>
			</head>
			<body class="container" style="background-color:#F0F0F0;overflow-y:hidden;padding-top:15px">
				<div class="col-lg-1 col-offset-6 centered">
					<form>
						<div class="form-check">
							<input class="form-check-input" type="checkbox" value="" id="rmPP">
							<label class="form-check-label" for="defaultCheck1">
								#{$strh['RemoveRefraction']}
							</label>
						</div>
						<div class="form-check">
							<input class="form-check-input" type="checkbox" value="" id="rmLEM">
							<label class="form-check-label" for="rmLEM">
								#{$strh['RemoveAllLEMs']}
							</label>
						</div>
						<div class="form-check">
							<input class="form-check-input" type="checkbox" value="" id="rmHiddenLEM">
							<label class="form-check-label" for="rmHiddenLEM">
								#{$strh['RemoveHiddenLEMs']}
							</label>
						</div>
						<div class="form-check">
							<input class="form-check-input" type="checkbox" value="" id="rmBur">
							<label class="form-check-label" for="rmBur">
								#{$strh['RemoveBlur']}
							</label>
						</div>
						<div class="form-check">
							<input class="form-check-input" type="checkbox" value="" id="rmBump">
							<label class="form-check-label" for="rmBump">
								#{$strh['RemoveBump']}
							</label>
						</div>
						<div class="form-check">
							<input class="form-check-input" type="checkbox" value="" id="rmAll">
							<label class="form-check-label" for="rmAll">
								#{$strh['RemoveAll']}
							</label>
						</div>
					</form>
					<nav class="navbar fixed-bottom navbar-light bg-light flex-row-reverse">
						<div>
							<button class="btn btn-light btn-sm mr-2" onclick='call_ruby1("ok_button")'>
								#{$strh['OK']}
							</button>
							<button class="btn btn-light btn-sm mr-2" onclick='call_ruby("cancel_button")'>
								#{$strh['Cancel']}
							</button>
						</div>
					</nav>
				</div>
			</body>
		</html>


		htmlString
		@@dialog.set_html(@htmlString)
		@@dialog.set_size(220, 240)

		@@dialog.add_action_callback( "ok_button" ) do |action_context, values|
			checkBoxValues = values.split(",")
			@@dialog.close() if handle_remove_selected(checkBoxValues)
		end
		
		@@dialog.add_action_callback( "cancel_button" ) do |action_context|
			@@dialog.close()
		end
		@@dialog.show
	end

	def reset_selected()
		if (!@refraction && !@lem & !@blur & !@bump & !@hiddenLEM &!@all)
			UI.messagebox("#{$strh['PleaseselectoneormorePodiummaterialpropertiestoremove']}")
			return false
		end
		
		if (!@all)
			Podium_Tools.new.reset_selected_materials(@refraction, @lem, @blur, @bump, @hiddenLEM)
		else
			reset_all()
		end
		return true
	end

	def reset_all()
		Podium_Tools.new.test_reset_materials_confirm
	end

	def handle_remove_selected( values )
		@refraction = values[0] == "true"
		@lem = values[1] == "true"
		@blur = values[2] == "true"
		@bump = values[3] == "true"
		@hiddenLEM = values[4] == "true"
		@all = values[5] == "true"
		
		reset_selected()
	end
end 

class ResetMaterialObserver
	def initialize( reset_dialog )
		@reset_dialog = reset_dialog 
	end

	def remove_selected( values )
		@reset_dialog.handle_remove_selected( values )
	end
end

file_loaded("Podium_Tools")